{% extends "modeladmin/index.html" %}
{% load i18n modeladmin_tags wagtailadmin_tags %}

{% block header_extra %}
    {% if not translations %}<div class="button button-secondary">{% endif %}
        {% include 'wagtailadmin/shared/locale_selector.html' with class='c-dropdown--large' theme="large" %}
    {% if not translations %}</div>{% endif %}
    {% if user_can_create %}
        {% include 'wagtail_localize/modeladmin/includes/button.html' with button=view.button_helper.add_button icon="plus" %}
    {% endif %}
    {% if view.list_export %}
        {% include view.export_buttons_template_name %}
    {% endif %}
{% endblock %}
