# Generated by Django 4.2.9 on 2024-01-06 13:20

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("wagtail_localize", "0015_translationcontext_field_path"),
    ]

    operations = [
        migrations.RenameField(
            model_name="translationlog",
            old_name="page_revision",
            new_name="revision",
        ),
    ]
