# Generated by Django 3.2.6 on 2021-09-14 08:53

from django.db import migrations


class Migration(migrations.Migration):
    dependencies = [
        ("wagtail_localize", "0013_translationsource_schema_version"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="translation",
            name="source_last_updated_at",
        ),
    ]
